#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
import subprocess
from sys import stderr, stdout
from subprocess import CalledProcessError


__license__   = 'GPL v3'
__copyright__ = u'2019, Jürgen Habelt <juergen@habelt-jena.de>'
__docformat__ = 'restructuredtext en'


import sys, os, re, platform
from os.path import expanduser
from calibre.utils.config import config_dir
from calibre_plugins.define_content.logging import LoggingFactory
from calibre_plugins.define_content.json import JsonConfigWrapper 


class Invoker(object):
    '''
    Responsible for invoking the Ruby script
    '''
    
    def __init__(self, settings):
        '''
        Initialization
        '''
        self.settings = settings
        self.gui = self.settings['gui'] # JsonConfigWrapper.getInstance()['gui']
        
        
    def invoke(self, book, result):
        '''
        Invokes the Ruby script to extract book marks
        '''
        cmd = self.build_command_line(book, result)
        return os.system(cmd)

    
    def invoke2(self, book, result):
        '''
        Invokes the Ruby script to extract book marks
        using the subprocess module
        '''
        args = self.build_command_args(book, result)
        LoggingFactory().getLogger(self).info('Invoking the Ruby interpreter')
        
        try:
            output = subprocess.check_output(args, universal_newlines = True)
            LoggingFactory().getLogger(self).info('Result from Ruby returned\n' + output)
        
        except CalledProcessError as ex:
            LoggingFactory().getLogger(self).exception('Exception during Ruby invocation: ' + ex.output)
            return ex.returncode
                
        return 0


    def rubyInstalled(self):
        '''
        Checks if Ruby is installed by calling the gem command
        '''
        ecode, output = self.commonInvoke(['gem' + self.systemShellCommandExtension(), '-v'])
        return ecode == 0
    
    
    def gemsInstalled(self):
        '''
        Checks if the required Ruby Gems: os and rubyzip are installed
        '''
        ecode, output = self.commonInvoke(['gem' + self.systemShellCommandExtension(), 'list'])
        if ecode != 0:
            return False
        
        match1 = re.search(r'^os ', output, re.MULTILINE)
        match2 = re.search(r'^rubyzip ', output, re.MULTILINE)
        return match1 and match2


    def pdftkInstalled(self):
        '''
        Checks if the configuration of pdftk references an existing file 
        '''
        pdftk_path = self.settings['pdftk']
        if not os.path.exists(pdftk_path):
            return False

        ecode, output = self.commonInvoke([pdftk_path, '--help'], False)
        return ecode == 0


    def customContentFieldExists(self):
        '''
        Checks whether the custom #content field exists
        '''
        if not self.gui:
            return False
        
        db = self.gui.current_db                        # query the #content custom field
        return '#content' in db.custom_field_keys()
        

    def commonInvoke(self, args, debug_output = True):
        '''
        Used to invoke a system command and get the exit code and the output back
        '''
        output = ''                                     # output defaults to empty string
        ecode = 0
        try:
            output = subprocess.check_output(args, universal_newlines = True)
            
        except CalledProcessError as ex:
            LoggingFactory().getLogger(self).exception('Exception in Common Invoke: ')
            ecode = ex.returncode
        
        except Exception as ex:
            LoggingFactory().getLogger(self).exception('Exception in Common Invoke: ')
            ecode = -1
        
        if debug_output:
            LoggingFactory().getLogger(self).debug('Common Invoke Result: (%d, %s)' % (ecode, output))
        
        return (ecode, output)
            
        
    def build_command_line(self, book, result):
        '''
        Builds the command line
        '''
        script_dir = os.path.join(config_dir, 'plugins', 'DefineContent', 'scripts')
        
        cmd = '%s' % self.settings['ruby']
        cmd += ' -C "%s"' % script_dir
        cmd += ' -I "%s"' % script_dir
        cmd += ' HtmlGenerator.rb'
        
        cmd += ' --path "%s"' % book
        cmd += ' --result "%s"' % result
        cmd += ' --tool "%s"' % self.settings['pdftk']
        cmd += ' --level %d' % self.settings['max_level']
        
        if self.settings['omit']:
            cmd += ' --omit'
        if self.settings['continuous']:
            cmd += ' --continuous'
        if self.settings['use_strip']:
            cmd += ' --strip "%s"' % self.settings['strip']
            
        cmd += ' >>"%s"' % self.settings['log_path']
            
        return cmd
                    

    def build_command_args(self, book, result):
        '''
        Builds the command arguments array
        '''
        script_dir = os.path.join(config_dir, 'plugins', 'DefineContent', 'scripts')
        
        args = [
            self.settings['ruby'],
            '-C', script_dir, 
            '-I', script_dir,
            'HtmlGenerator.rb',
            '--path', book,
            '--result', result,
            '--tool', self.settings['pdftk'],
            '--level', str(self.settings['max_level'])
        ]

        if self.settings['omit']:
            args += '--omit'
        if self.settings['continuous']:
            args += '--continuous'
        if self.settings['use_strip']:
            args += ['--strip', self.settings['strip']]
        
        return args
    
    
    def systemShellCommandExtension(self):
        '''
        Gem maybe other than .cmd in Linux
        '''
        if platform.system() == 'Windows':
            return '.cmd'
        else:
            return ''


    def systemNullDevice(self):
        '''
        To supress command output independently of system
        '''
        if platform.system() == 'Windows':
            return 'nul'
        else:
            return '/dev/null'
        