require 'erb'
require 'optparse'
require 'optparse/time'
require 'ostruct'
require 'pp'

require 'Utils'
require 'Bookmark'
require 'PdfExtractor'
require 'EpubExtractor'
require 'LoggerWrapper'


# Responsible to generate a html document from either a pdf or epub e-book
class HtmlGenerator

  PDF_TOOL = 'C:\Programme (extra)\PDFtk Server\bin\pdftk.exe'
  TEMPLATES = "templates"
  
  # The initialization method
  def initialize()
    @options = nil
    @extractors = { '.pdf' => PdfExtractor, '.epub' => EpubExtractor }
  end

  include Utils
    
  # Generate a list inside the HMTL document
  def generate_list( bookmark )
    template_path = File.join(TEMPLATES, 'Content.template')
    text = File.new( template_path ).read                                   # reads the template text
    generator = ERB.new( text )                                             # the template system
    result = generator.result( binding )                                    # generate the result
    result 
  end

  # Generate the HTML document
  def generate_html( bookmark )
    base = File.basename( path, '.*' )
    LoggerWrapper.instance().info('Ruby')  { "Generating from #{base}" }
  
    result_data = generate_list( bookmark )
    
    File.open( result, 'w') do | file |
      file.write( result_data )                                             # finally write
    end
    LoggerWrapper.instance().info('Ruby') { "File from #{base} generated" }
  end
  

  # The main method to generate a html document from pdf/epub bookmarks
  # path - path of the pdf/epub document
  # max_level - the maximum level to put out
  # continuous - flag, if level 2 items should be numbered continuously
  # omit - flag, if html numbering should be omitted
  def generate( )
    ext = File.extname( path )
    extractor_class = @extractors[ext]
    extractor = extractor_class.new( @options )
    to = extractor.extract( path )
    root = extractor.parse( to )
    root.to_log()
    generate_html( root )
  end
  
  
  # Parses the command line
  def parse_cmd_line( args )
    @options = OpenStruct.new
    @options.path = nil
    @options.max_level = 2
    @options.continuous = false
    @options.omit = false
    @options.skip = false
    @options.strip = /^/
    @options.result = nil
    @options.pdf_tool = nil

    begin
      opt_parser = OptionParser.new do | opts |
        opts.banner = "Usage: HtmlGenerator.rb [options]"
  
        opts.separator ""
        opts.separator "Specific options:"
        
        # Mandatory argument.
        opts.on("-p DOCUMENT-PATH", "--path DOCUMENT-PATH",
                "Requires the document path") do | path |
          @options.path = path
        end
        
        opts.on("-r RESULT-PATH", "--result RESULT-PATH",
                "Specify result path (default DOCUMENT-PATH.html)") do | result |
          @options.result = result
        end
        
        opts.on("-t PDF-TOOL", "--tool PDF-TOOL",
                "Specify pdf tool (default PDF Tool on this machine)") do | pdf_tool |
          @options.pdf_tool = pdf_tool
        end
  
        # Cast to integer.
        opts.on("-l INTEGER", "--level INTEGER", OptionParser::DecimalInteger,
                "Specify maximum level (default 2)") do | level |
          @options.max_level = level
        end
        
        # Boolean switch.
        opts.on("-c", "--[no-]continuous", "Count level 2 bookmarks continuously") do | c |
          @options.continuous = c
        end
        
        # Boolean switch.
        opts.on("-o", "--[no-]omit", "Omit ordered list numbers") do | o |
          @options.omit = o
        end
        
        # Boolean switch.
        opts.on("-S", "--[no-]skip", "Skip first level headings") do | s |
          @options.skip = s
        end
        
        # Strip Regex.
        opts.on("-s [REGEXP]", "--strip [REGEXP]",
                "Specify regular expression (default /^ *\d*\.?\d* */) to strip title from the beginning") do | s |
          if (s)
            @options.strip = eval( s )
          else
            @options.strip = /^ *\d*\.?\d* */
          end
        end
  
        opts.separator ""
        opts.separator "Common options:"
  
        # No argument, shows at tail.  This will print an options summary.
        # Try it and see!
        opts.on_tail("-h", "--help", "Show this message") do
          puts opts
          exit
        end
      end
  
      opt_parser.parse!(args)
      if (! @options.path)
        raise OptionParser::MissingArgument, "No document path specified"
      end
      if (! @options.result)
        @options.result = change_extension( @options.path, '.html' )
      end
      if (! @options.pdf_tool)
        @options.pdf_tool = PDF_TOOL
      end
    rescue Exception => ex
      puts opt_parser.help()
      puts
      raise
    end
    @options
  end


  # The main method using parameters from the command line  
  def self.generate()
    begin
      generator = HtmlGenerator.new()
      options = generator.parse_cmd_line( ARGV )
      LoggerWrapper.instance().info('Ruby') { options.pretty_inspect() }
      generator.generate( )
    
    rescue Exception => ex
      LoggerWrapper.instance().fatal('Ruby') { ex }
      exit(1)
    end
    exit(0)
  end
    
end


HtmlGenerator.generate()
